@@ -40,7 +40,6 @@
 #endif
 #include <time.h>
 
-#include "support.h"
 #include "metadata.h"
 #include "interface.h"
 #include "callbacks.h"
@@ -49,7 +48,7 @@
 #include "localisation.h"
 
 #define APP_NAME "conboy"
-#define APP_VER "0.2.0"
+#define APP_VER "0.2.1"
 #define APP_SERVICE "de.zwong.conboy"
 #define APP_METHOD "/de/zwong/conboy"
 
@@ -62,7 +61,7 @@ int
 main (int argc, char *argv[])
 {
   HildonProgram *program;
-  Note *note; /*= g_slice_alloc0(sizeof(Note));*/
+  Note *note;
   osso_context_t *osso_context;
   AppData *app_data;
 
@@ -88,7 +87,7 @@ main (int argc, char *argv[])
   if (app_data->all_notes != NULL && app_data->all_notes->data != NULL) {
 	  note = (Note*)app_data->all_notes->data;
   } else {
-	  note = g_new0(Note, 1);
+	  note = note_create_new();
   }
   
   note_show(note);
